module Ecoportal
  module API
    class V2
      class Page < Common::Content::DoubleModel
        ALLOWED_KEYS  = %w[id patch_ver name template_id base_tags tags time_zone created_at updated_at can components sections stages]
        passkey     :id
        passthrough :patch_ver
        passthrough :name, :template_id
        passarray   :base_tags, :tags, order_matters: false
        passthrough :time_zone
        passdate    :created_at, :updated_at, read_only: true
        passthrough :can

        class_resolver :components_class, "Ecoportal::API::V2::Page::Components"
        class_resolver :sections_class,   "Ecoportal::API::V2::Page::Sections"
        class_resolver :stages_class,     "Ecoportal::API::V2::Page::Stages"

        embeds_many :components, enum_class: :components_class
        embeds_many :sections,   enum_class: :sections_class
        embeds_many :stages,     enum_class: :stages_class

        def as_update
          super.tap do |hash|
            unless !hash
              hash["data"].select! do |key, value|
                ALLOWED_KEYS.include?(key)
              end
            end
          end
        end

      end
    end
  end
end

require 'ecoportal/api/v2/page/component'
require 'ecoportal/api/v2/page/components'
require 'ecoportal/api/v2/page/section'
require 'ecoportal/api/v2/page/sections'
require 'ecoportal/api/v2/page/stage'
require 'ecoportal/api/v2/page/stages'
