module Ecoportal
  module API
    class V2
      class Page
        class Section < Common::Content::DoubleModel
          class << self
            def new_doc(split: false)
              {
                "id"     => new_uuid,
                "type"   => split ? "split" : "content",
                "weight" => 9999
              }.tap do |out|
                component_ids = if split
                    {
                      "left_component_ids" => [],
                      "right_component_ids" => []
                    }
                  else
                    {
                      "component_ids" => []
                    }
                  end
                out.merge!(component_ids)
              end
            end
          end

          passkey     :id
          passthrough :patch_ver, :weight, :type
          passthrough :heading, :left_heading, :right_heading
          passarray   :component_ids, :left_component_ids, :right_component_ids
          passthrough :minimized

          def split?
            doc && doc["type"] == "split"
          end

          def all_component_ids
            return component_ids.to_a unless split?
            left_component_ids.to_a | right_component_ids.to_a
          end

          def component?(id)
            all_component_ids.include?(id)
          end

          def components
            fld_ids = all_component_ids
            root.components.values_at(*fld_ids).select.with_index do |fld, i|
              puts "Warning: field #{id} points to missing field #{fld_ids[i]}" if !fld
              fld && (!block_given? || yield(fld))
            end
          end

        end
      end
    end
  end
end
